from distutils.core import setup

setup(
    name='pyclictk',
    version='0.2',
    packages=['clictk'],
    provides=['clictk'],
    url='http://github.com/CognitionGuidedSurgery/pyclictk',
    license='GPLv3',
    author='Alexander Weigl',
    author_email='uiduw@student.kit.edu',
    description='Python Support Common Toolkit\'s (CTK) Command Line Interface (CLI)'
)
